<?php

namespace Drupal\diff_translation\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines dynamic local tasks.
 */
class DynamicLocalTasks extends DeriverBase implements ContainerDeriverInterface {

  use StringTranslationTrait;

  /**
   * Construct the diff translation local tasks.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   */
  public function __construct(
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected readonly LanguageManagerInterface $languageManager,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('language_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition): array {
    $languages = $this->languageManager->getLanguages();
    if (count($languages) < 2) {
      return parent::getDerivativeDefinitions($base_plugin_definition);
    }

    $default_language = $this->languageManager->getDefaultLanguage();
    $first_language = $languages[$default_language->getId()];
    unset($languages[$default_language->getId()]);
    $second_language = array_pop($languages);

    $definitions = $this->entityTypeManager->getDefinitions();
    foreach ($definitions as $entity_type => $definition) {
      if ($definition->isRevisionable()
        && $definition->isTranslatable()
        && $definition->entityClassImplements(ContentEntityInterface::class)
        && $definition->hasRouteProviders()
      ) {
        $task_name = 'diff_translation.' . $entity_type;
        $this->derivatives[$task_name] = $base_plugin_definition;
        $this->derivatives[$task_name]['title'] = $this->t('Compare translations')->render();
        $this->derivatives[$task_name]['route_name'] = 'entity.' . $entity_type . '.diff_translation';
        $this->derivatives[$task_name]['base_route'] = 'entity.' . $entity_type . '.canonical';
        $this->derivatives[$task_name]['route_parameters'] = [
          'left_lang' => $first_language->getId(),
          'right_lang' => $second_language->getId(),
        ];
      }
    }

    return parent::getDerivativeDefinitions($base_plugin_definition);
  }

}
