<?php

namespace Drupal\diff_translation\Plugin\diff\Layout;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Url;
use Drupal\diff_translation\DiffEntityTranslationComparison;
use Drupal\diff_translation\DiffEntityTranslationParser;
use Drupal\diff\Plugin\diff\Layout\SplitFieldsDiffLayout;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides Split fields diff layout.
 *
 * @DiffLayoutBuilder(
 *   id = "split_fields_diff_translation",
 *   label = @Translation("Split fields diff translation"),
 *   description = @Translation("Field based layout, displays revision comparison side by side."),
 * )
 */
class SplitFieldsDiffTranslationLayout extends SplitFieldsDiffLayout {

  /**
   * Constructs a SplitFieldsDiffLayout object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config
   *   The configuration factory object.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\diff_translation\DiffEntityTranslationParser $entity_parser
   *   The entity parser.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date
   *   The date service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param \Drupal\diff_translation\DiffEntityTranslationComparison $entity_comparison
   *   The diff entity comparison service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ConfigFactoryInterface $config,
    EntityTypeManagerInterface $entity_type_manager,
    DiffEntityTranslationParser $entity_parser,
    DateFormatterInterface $date,
    RendererInterface $renderer,
    DiffEntityTranslationComparison $entity_comparison,
    RequestStack $request_stack,
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $config,
      $entity_type_manager,
      $entity_parser,
      $date,
      $renderer,
      $entity_comparison,
      $request_stack
    );

    $this->renderer = $renderer;
    $this->entityComparison = $entity_comparison;
    $this->requestStack = $request_stack;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('diff_translation.entity_parser'),
      $container->get('date.formatter'),
      $container->get('renderer'),
      $container->get('diff_translation.entity_comparison'),
      $container->get('request_stack'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build(
    ContentEntityInterface $left_revision,
    ContentEntityInterface $right_revision,
    ContentEntityInterface $entity
  ): array {
    $this->requestStack->getCurrentRequest()->query->set('filter', 'strip_tags');
    $build = parent::build($left_revision, $right_revision, $entity);
    unset($build['header']);
    unset($build['controls']);

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  protected function buildTableHeader(ContentEntityInterface $left_revision, ContentEntityInterface $right_revision): array {
    $left_languages = $left_revision->getTranslationLanguages();
    $right_languages = $right_revision->getTranslationLanguages();
    $left_language = $left_revision->language();
    $right_language = $right_revision->language();
    $entity_type = $left_revision->getEntityTypeId();

    $header = [
      'left' => [
        'colspan' => 3,
        'data' => [
          '#type' => 'operations',
          '#links' => [
            [
              'title' => $left_language->getName(),
              'url' => new Url('entity.' . $entity_type . '.diff_translation', [
                $entity_type => $left_revision->id(),
                'left_lang' => $left_language->getId(),
                'right_lang' => $right_language->getId(),
              ]),
            ]
          ]
        ],
      ],
      'right' => [
        'colspan' => 3,
        'data' => [
          '#type' => 'operations',
          '#links' => [
            [
              'title' => $right_language->getName(),
              'url' => new Url('entity.' . $entity_type . '.diff_translation', [
                $entity_type => $right_revision->id(),
                'left_lang' => $right_language->getId(),
                'right_lang' => $left_language->getId(),
              ]),
            ]
          ]
        ],
      ]
    ];

    foreach ($right_languages as $language) {
      if ($language->getId() == $left_language->getId()) {
        continue;
      }
      $header['left']['data']['#links'][] = [
        'title' => $language->getName(),
        'url' => new Url('entity.' . $entity_type . '.diff_translation', [
          $entity_type => $left_revision->id(),
          'left_lang' => $language->getId(),
          'right_lang' => $right_revision->language()->getId(),
        ]),
        '#access' => ($language->getId() != $left_revision->language()->getId()),
      ];
    }

    foreach ($left_languages as $language) {
      if ($language->getId() == $right_language->getId()) {
        continue;
      }
      $header['right']['data']['#links'][] = [
        'title' => $language->getName(),
        'url' => new Url('entity.' . $entity_type . '.diff_translation', [
          $entity_type => $left_revision->id(),
          'right_lang' => $language->getId(),
          'left_lang' => $left_revision->language()->getId(),
        ]),
      ];
    }

    return $header;
  }

}
