<?php

namespace Drupal\diff_translation\Routing;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Contains routes for diff functionality.
 */
class DiffTranslationRoutes implements ContainerInjectionInterface {

  /**
   * Construct the diff translation routes.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(
    protected readonly EntityTypeManagerInterface $entityTypeManager,
  ) {

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Dynamically defines the entity comparison routes.
   *
   * @return \Symfony\Component\Routing\RouteCollection
   */
  public function routes(): RouteCollection {
    $collection = new RouteCollection();
    $definitions = $this->entityTypeManager->getDefinitions();
    foreach ($definitions as $entity_type => $definition) {
      if ($definition->isRevisionable()
        && $definition->isTranslatable()
        && $definition->entityClassImplements(ContentEntityInterface::class)
        && $definition->hasRouteProviders()
      ) {
        $collection->add('entity.' . $entity_type . '.diff_translation', $this->getRoute($definition));
      }
    }

    return $collection;
  }

  /**
   * Constructs the diff route.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type.
   *
   * @return \Symfony\Component\Routing\Route
   *   The diff route.
   */
  protected function getRoute(EntityTypeInterface $entity_type): Route {
    $route = new Route("/{$entity_type->id()}/{{$entity_type->id()}}/diff_translation/{left_lang}/{right_lang}");
    $route->addDefaults([
      '_controller' => '\Drupal\diff_translation\Controller\DiffTranslationController::compareEntityTranslation',
    ])->addRequirements([
      '_entity_access' => $entity_type->id() . '.view',
    ])->setOption('parameters', [
      $entity_type->id() => ['type' => 'entity:' . $entity_type->id()],
    ])->setOption('_admin_route', TRUE);

    return $route;
  }

}
